#version 330 core

const int MAX_POINT_LIGHTS = 2; 

in vec2 fragmentUV;
in vec3 fragmentNormal;
in vec3 worldPos;

layout(location=0) out vec4 fragColor;

struct BaseLight                                                                    
{                                                                                   
    vec3 Color;                                                                     
    float AmbientIntensity;                                                         
    float DiffuseIntensity;                                                         
};                                                                                  
                                                                                    
struct DirectionalLight                                                             
{                                                                                   
    BaseLight Base;                                                                 
    vec3 Direction;                                                                 
};                                                                                  
                                                                                    
struct Attenuation                                                                  
{                                                                                   
    float Constant;                                                                 
    float Linear;                                                                   
    float Exp;                                                                      
};                                                                                  
                                                                                    
struct PointLight                                                                           
{                                                                                           
    BaseLight Base;                                                                         
    vec3 Position;                                                                          
    Attenuation Atten;                                                                      
};

struct SpotLight                                                                            
{                                                                                           
    PointLight Base;                                                                        
    vec3 Direction;                                                                         
    float Cutoff;                                                                           
};    

uniform sampler2D colorTexture;
uniform DirectionalLight directionalLight;
uniform vec3 eyeWorldPos;
uniform float matSpecularIntensity;
uniform float specularPower;
uniform int numPointLights; 
uniform PointLight pointLights[MAX_POINT_LIGHTS];  
uniform SpotLight spotLight;




vec4 CalcLightInternal(BaseLight light, vec3 lightDirection, vec3 normal)                   
{                                                                                           
    vec4 ambientColor = vec4(light.Color, 25.0f) * light.AmbientIntensity;                   
    float diffuseFactor = dot(normal, -lightDirection);                                     
                                                                                            
    vec4 diffuseColor  = vec4(0, 0, 0, 0);                                                  
    vec4 specularColor = vec4(0, 0, 0, 0);                                                  
                                                                                            
    if (diffuseFactor > 0) {                                                                
        diffuseColor = vec4(light.Color, 1.0f) * light.DiffuseIntensity * diffuseFactor;    
                                                                                            
        vec3 vertexToEye = normalize(eyeWorldPos - worldPos);                             
        vec3 lightReflect = normalize(reflect(lightDirection, normal));                     
        float specularFactor = dot(vertexToEye, lightReflect);                              
        specularFactor = pow(specularFactor, specularPower);                               
        if (specularFactor > 0) {                                                           
            specularColor = vec4(light.Color, 1.0f) *                                       
                            matSpecularIntensity * specularFactor;                         
        }                                                                                   
    }                                                                                       
                                                                                            
    return (ambientColor + diffuseColor + specularColor);                                   
}     

vec4 CalcDirectionalLight(vec3 normal)                                                      
{                                                                                           
    return CalcLightInternal(directionalLight.Base, directionalLight.Direction, normal); 
}             

vec4 CalcPointLight(PointLight l, vec3 normal)                                                 
{                                                                                           
    vec3 lightDirection = worldPos - l.Position;                         
    float distance = length(lightDirection);                                                
    lightDirection = normalize(lightDirection);                                             
                                                                                            
    vec4 color = CalcLightInternal(l.Base, lightDirection, normal);       
    float attenuation =  l.Atten.Constant +                               
                         l.Atten.Linear * distance +                      
                         l.Atten.Exp * distance * distance;               
                                                                                            
    return color / attenuation;                                                             
}              

vec4 CalcSpotLight(SpotLight l, vec3 normal)
{
    vec3 lightToPixel = normalize(worldPos - l.Base.Position);
    float spotFactor = dot(lightToPixel, l.Direction);

    if (spotFactor > l.Cutoff) {
        vec4 color = CalcPointLight(l.Base, normal);
        return color * (1.0 - (1.0 - spotFactor) * 1.0/(1.0 - l.Cutoff));
    }
    else {
        return vec4(0,0,0,0);
    }
}    

void main()
{
	vec3 normal = normalize(fragmentNormal);                                                       
    vec4 totalLight = CalcDirectionalLight(normal);                                         
                                                                                            
    for (int i = 0 ; i < numPointLights ; i++) {                                           
        totalLight += CalcPointLight(pointLights[i], normal);                                            
    }                                                                                       
    totalLight += CalcSpotLight(spotLight, normal);
	                                                                         
    fragColor = texture2D(colorTexture, fragmentUV.xy) * totalLight;
}